<?php
/**
 * @package   Sessionize.com
 * @author    Sessionize.com
 * @link      http://sessionize.com/
 *
 * Plugin Name:       Sessionize Schedule Embed
 * Description:       Embed schedule from Sessionize
 * Version:           2.0.1
 * Author:            Sessionize.com
 * Author URI:        http://sessionize.com/
 * Text Domain:       sessionize
 */

class Sessionize{

	/**
	 * Instance of this class.
	 *
	 * @since    1.0.0
	 *
	 * @var      object
	 */
	protected static $instance = null;

	/**
	 * Sessionize API endpoint
	 *
	 * @since    1.0.0
	 *
	 */
	private $remote_embed_url = 'https://sessionize.com/api/v2/[ID]/view/[EMBED_TYPE]/?js=external';
	private $remote_embed_types_url = 'https://sessionize.com/api/v2/meta/wordpress';


	/**
	 * Initialize the plugin
	 *
	 * @since     1.0.0
	 */
	private function __construct() {

		add_shortcode( 'sessionize', array( $this, 'sessionize_shortcode_handler' ) );
		add_action( 'admin_head', array( $this, 'sessionize_editor_button' ) );
		add_action( 'after_wp_tiny_mce', array( $this, 'tinymce_extra_vars' ) );
		add_action( 'init', array( $this, 'sessionize_gb_block' ) );

	
	}

	/**
	 * Return an instance of this class.
	 *
	 * @since     1.0.0
	 *
	 * @return    object    A single instance of this class.
	 */
	public static function get_instance() {

		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

	/**
	 * Sessionize gutenberg block enqueue scripts
	 *
	 * @since     2.0.0
	 *
	 */
	public function sessionize_gb_block() {
		// Scripts.
		wp_enqueue_script(
			'sessionize-embed-block',
			plugins_url( 'assets/scripts/block.js', __FILE__ ),
			array( 'wp-blocks', 'wp-element', 'wp-data', 'wp-components', 'wp-editor', 'wp-server-side-render' ),
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/scripts/block.js' )
		);

		register_block_type( 'sessionize/embed-block', array(
	        'editor_script' => 'sessionize-embed-block',
	        'attributes' => array(
				'embed_id' => array('type' => 'string'),
				'embed_type' => array('type' => 'string')
			),
	        'render_callback' => array( $this, 'sessionize_embed_block_render_callback' )
	    ) );

		$sessionize_g_block_object = $this->sessionize_js_object($label = 'label');
		wp_localize_script( 'sessionize-embed-block', 'sessionize_g_block_object', $sessionize_g_block_object );

		// Styles.
		wp_enqueue_style(
			'sessionize-embed-block-editor',
			plugins_url( 'assets/style/editor.css', __FILE__ ),
			array( 'wp-edit-blocks' ),
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/style/editor.css' )
		);
	}

	/**
	 * Sessionize gutenberg block callback
	 *
	 * @since     2.0.0
	 *
	 */
	public function sessionize_embed_block_render_callback( $attributes, $content ) {
		$embed_id = $attributes['embed_id'];
		$embed_type = $attributes['embed_type'];

		$is_gb_editor = \defined( 'REST_REQUEST' ) && REST_REQUEST && ! empty( $_REQUEST['context'] ) && 'edit' === $_REQUEST['context'];

		if( $is_gb_editor){
			$content = '<div class="sessionize-block">';
			$content .= '<img src="'.plugins_url( 'assets/images/logo--symbol.svg', __FILE__ ).'" class="logo"/>';
			$content .= '<p><strong>'.__('Sessionize Embed', 'sessionize').'</strong></p>';
			$content .= '<p><i>'.__('Configure in side panel. Preview or update to see it live.', 'sessionize').'</i></p>';
			$content .= '<div>';
			return $content;
		}

		return $this->sessionize_shortcode_handler(array('id' => $embed_id, 'embed_type' => $embed_type ));
	}

	/**
	 * Sessionize shortcode handler
	 *
	 * @since     1.0.0
	 *
	 */
	public function sessionize_shortcode_handler( $atts ) {

		$shortcode_atts = shortcode_atts( array(
		    'id' => '',
		    'embed_type' => '',
		), $atts );

		if ( empty( $shortcode_atts['id'] ) || empty( $shortcode_atts['embed_type'] ) )
			return;

		$event_id = $shortcode_atts['id'];
		$embed_type = $shortcode_atts['embed_type'];

		if( false === ( $embed_output = get_transient( 'sessionize_embed_'.$event_id.'_'.$embed_type ) ) ){

			$remote_embed_url =  $this->remote_embed_url;
			$remote_embed_url =  str_replace('[ID]', $event_id, $remote_embed_url);
			$remote_embed_url =  str_replace('[EMBED_TYPE]', $embed_type, $remote_embed_url);

			$embed = wp_remote_get( $remote_embed_url );

			$embed_output = wp_remote_retrieve_body( $embed );
			$response_code = wp_remote_retrieve_response_code( $embed );

			if( $response_code === 200 ){
				set_transient( 'sessionize_embed_'.$event_id.'_'.$embed_type, $embed_output, 10 * MINUTE_IN_SECONDS );
			}
		}

		return $embed_output;

	}

	/**
	 * Editor button hooks
	 *
	 * @since     1.0.0
	 *
	 */
	function sessionize_editor_button() {
	    
	    if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
	        return;
	    }

	    add_filter( 'mce_external_plugins', array( $this, 'add_buttons' ) );
	    add_filter( 'mce_buttons', array( $this, 'register_buttons' ) );
	}

	/**
	 * Register button popup
	 *
	 * @since     1.0.0
	 *
	 */
	public function add_buttons( $plugin_array ) {
	    
	    $plugin_array['sessionize_button'] = plugins_url( 'assets/scripts/tinymce_button.js', __FILE__ );
	    return $plugin_array;
	
	}

	/**
	 * Register button
	 *
	 * @since     1.0.0
	 *
	 */
	public function register_buttons( $buttons ) {
	    
	    array_push( $buttons, 'sessionize_button' );
	    return $buttons;
	
	}

	/**
	 * Fetch sessionize embed types
	 *
	 * @since     1.0.0
	 *
	 */
	private function remote_embed_types($label){
		
		$embed_types = array();

		if ( false === ( $embed_types = get_transient( 'sessionize_embed_types_'.$label ) ) ) {

			$remote_embed_types_url = $this->remote_embed_types_url;
			
			$remote_embed_types = wp_remote_get( esc_url_raw( $remote_embed_types_url ) );

			if ( !is_wp_error( $remote_embed_types ) ) {
				$remote_embed_types = json_decode( wp_remote_retrieve_body( $remote_embed_types ), true );
			}

			if( is_array( $remote_embed_types ) ){
				foreach( $remote_embed_types as $remote_embed_type_key => $remote_embed_type_value ){
					$embed_types[] = array(
						$label => $remote_embed_type_value,
						'value' => $remote_embed_type_key,
					);
				}
			}

			set_transient( 'sessionize_embed_types_'.$label, $embed_types, 6 * HOUR_IN_SECONDS );
		}

		return $embed_types;
	}

	/**
	 * Sessionize tinyMCE object
	 *
	 * @since     1.0.0
	 *
	 */
	public function tinymce_extra_vars() { ?>
		<script type="text/javascript">
			var sessionize_tinyMCE_object = <?php echo json_encode( $this->sessionize_js_object($label = 'text') ); ?>;
		</script><?php
	}

	/**
	 * Sessionize general JS object
	 *
	 * @since     2.0.0
	 *
	 */
	public function sessionize_js_object($label){
		return array(
			'button_name' 		=> esc_html__( '', 'sessionize' ),
			'sessionize_icon' 	=> plugins_url( 'assets/images/sessionize-icon-filled.png', __FILE__ ),
			'button_title' 		=> esc_html__( 'Sessionize Embed', 'sessionize' ),
			'event_id_title' 	=> esc_html__( 'Your embed ID from Sessionize', 'sessionize' ),
			'event_id_error_msg' 	=> esc_html__( 'Please enter embed ID from Sessionize', 'sessionize' ),
			'embed_type_title' 	=> esc_html__( 'View', 'sessionize' ),
			'embed_type_title_block' 	=> esc_html__( 'Select View', 'sessionize' ),
			'embed_type_msg' 	=> esc_html__( 'Select the view', 'sessionize' ),
			'embed_types' 		=> $this->remote_embed_types($label),
		);
	}
}

Sessionize::get_instance();